/* $Id: //guest/paul_krause/perforce/integrations/jdev/src/com/perforce/jdev/P4Dir.java#5 $
 * Copyright 2000-2002 SoundBite Corporation.   All Rights Reserved.
 */
package com.perforce.jdev;

import com.perforce.api.Change;
import com.perforce.api.DirEntry;
import com.perforce.api.Env;
import com.perforce.api.FileEntry;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import javax.swing.Icon;
import oracle.ide.scm.SCMFile;
import oracle.ide.scm.SCMFileStatus;
import oracle.ide.scm.SCMOptions;
import oracle.ide.scm.SCMSystem;
import oracle.ide.scm.env.SCMFileFactory;
import oracle.ide.scm.error.SCMException;
import oracle.ide.scm.util.SCMAssert;
import com.perforce.api.SourceControlObject;

/**
 * @author Paul Krause
 * @created Thu Apr 25 12:29:17 2002
 * @version $Revision: #5 $
 */

public class P4Dir extends P4Node {

   private static SCMFile createSCMFile (DirEntry de) {
      final SCMFileFactory util = SCMSystem.env().getFileFactory();
      return util.createFile(de.getPath());
   }

   /** The IDE-neutral representaion of a directory in depot */
   private final DirEntry dirEntry;

   /**
    * Constructs a P4Node for a DirEntry
    *
    * @param dir An IDE-neutral representaion of a depot directory
    */
   P4Dir(DirEntry dir)
   {
      super(createSCMFile(dir), dir);
      dirEntry = dir;
   }

   /**
    * Constructs a P4Node and DirEntry for an SCMFile
    *
    * @param f Client SCMFile
    * @param env Perforce environment
    */
   P4Dir(SCMFile f, Env env)
   {
      this(DirEntry.getDirEntry(env, f.getPath(), true));
   }

   /**
    * Gets the depot entry for use by superclass.
    *
    * @return a <code>DirEntry</code> object
    */
   protected SourceControlObject getP4Object() {
      return dirEntry;
   }

   /**
    * Gets the value of the locked property
    *
    * @return The Locked value
    */
   boolean isLocked()
   {
      return false;
   }

   /**
    * Description of the Method
    *
    * @param a Description of Parameter
    * @param b Description of Parameter
    * @param c Description of Parameter
    * @exception IOException Description of Exception
    */
   void delete(boolean a, boolean b, Change c)
      throws IOException
   {
      veto("Vetoed attempt to delete all files in directory");
   }

   /**
    * Description of the Method
    *
    * @exception SCMException Description of Exception
    */
   void openForEdit(SCMOptions opts)
      throws SCMException
   {
      throw new SCMException("directories are not controlled");
   }

   /**
    * Gets the value of the env property
    *
    * @return The Env value
    */
   protected Env getEnv()
   {
      return dirEntry.getEnv();
   }

   /**
    * Sets the value of the URL property
    *
    * @param loc The new URL value
    */
   public void setURL(URL loc) { }

   /**
    * Gets the value of the icon property
    *
    * @return The Icon value
    */
   public Icon getIcon()
   {
      return null;
   }

   /**
    * Gets the value of the URL property
    *
    * @return The URL value
    */
   public URL getURL()
   {
      try {
         return toFile().toURL();
      } catch (MalformedURLException e) {
         SCMAssert.fail(e);
         throw new IllegalStateException(e.getMessage());
      }
   }

   /**
    * Gets the value of the directory property
    *
    * @return The Directory value
    */
   public boolean isDirectory()
   {
      return true;
   }

   /**
    * Gets the value of the latest status property
    *
    * @return The LatestStatus value
    * @exception SCMException Description of Exception
    */
   public SCMFileStatus getLatestStatus()
      throws SCMException
   {
      dirEntry.sync();
      return this;
   }

   /**
    * Gets the value of the file name property
    *
    * @return The FileName value
    */
   public String getFileName()
   {
      return dirEntry.getBasePath();
   }

   /**
    * Gets the value of the name property
    *
    * @return The Name value
    */
   public String getName()
   {
      return toFile().getName();
   }

   /**
    * Gets the value of the path property
    *
    * @return The Path value
    */
   public String getPath()
   {
      return dirEntry.getPath();
   }

   /**
    * Gets the value of the extension property
    *
    * @return The Extension value
    */
   public String getExtension()
   {
      return null;
   }

   /**
    * Gets the value of the parent property
    *
    * @return The Parent value
    */
   public SCMFile getParent()
   {
      final DirEntry pe = dirEntry.getParent();
      return new P4Dir(pe);
   }

   /**
    * Gets the value of the workspace property
    *
    * @return The Workspace value
    */
   public boolean isWorkspace()
   {
      return false;
   }

   /**
    * Gets the value of the project property
    *
    * @return The Project value
    */
   public boolean isProject()
   {
      return false;
   }

   /**
    * Gets the value of the project sources property
    *
    * @return The ProjectSources value
    */
   public Iterator getProjectSources()
   {
      return null;
   }

   /**
    * Gets the value of the data property
    *
    * @return The Data value
    */
   public Object getData()
   {
      return null;
   }

   /**
    * Gets the value of the overlay icon property
    *
    * @return The OverlayIcon value
    */
   public Icon getOverlayIcon()
   {
      return null;
   }

   /**
    * Gets the value of the versioned property
    *
    * @return The Versioned value
    */
   public boolean isVersioned()
   {
      return false;
   }

   /**
    * Gets the value of the tool tip text property
    *
    * @return The ToolTipText value
    */
   public String getToolTipText()
   {
      return dirEntry.getPath();
   }

   /**
    * Gets the value of the input stream property
    *
    * @return The InputStream value
    */
   public InputStream getInputStream()
   {
      FileEntry[] files = dirEntry.getFiles();

      return null;
   }

   /**
    * Gets the value of the category property
    *
    * @return The Category value
    */
   public int getCategory()
   {
      return 0;
   }

   /**
    * Description of the Method
    *
    * @return Description of the Returned Value
    */
   public boolean exists()
   {
      return true;
   }

   /**
    * Description of the Method
    *
    * @param f Description of Parameter
    * @return Description of the Returned Value
    */
   public boolean hasParent(SCMFile f)
   {
      return (f instanceof P4Dir)
         && f.toFile().equals(toFile().getParentFile());
   }

   /**
    * Description of the Method
    *
    * @exception IOException Description of Exception
    */
   public void open()
      throws IOException
   {
      // TODO:  Override this com.perforce.jdev.P4Node method
      super.open();
   }

   /**
    * Description of the Method
    *
    * @exception IOException Description of Exception
    */
   public void close()
      throws IOException
   {
      // TODO:  Override this com.perforce.jdev.P4Node method
      super.close();
   }

}
